var IteratorGraph = (function () {
    'use strict';
    /**
     * @description Crée un itérateur pour parcourir les éléments données.
     * @param {Object|Array} element Eléments à parcourir.
     * @typedef IteratorGraph Un itérateur capable de parcourir des Object|Array.
     */
    function IteratorGraph (element) {
        /**
         * @description Eléments à parcourir. Peut être un Object ou un Array.
         * @type Object|Array
         */
        var _element = element;
        /**
         * @description Clé de l'objet à parcourir.
         * @type Array<String>|undefined
         */
        var _keys;
        /**
         * @description Index utilisé pour le parcours des éléments.
         * @type Number
         */
        this.index = 0;
        /**
         * @description Donne l'élément à l'index donné.
         * @param index
         * @returns {*}
         */
        this.getElementAt = function (index) {
            if (_keys) return _element[_keys[index]];
            else       return _element[index];
        };
        /**
         * @description Donne la taille de _element.
         * @returns {Number}
         */
        this.size = function () {
            if (_keys) return _keys.length;
            else       return _element.length;
        };

        /* Uniquement si l'objet à parcourir n'est pas un tableau. */
        if (!_element.length && _element.length !== 0) {
            _keys = Object.keys(_element);
        }

        /**
         * @description Re-initialize this IteratorGraph with the new elements.
         * @param {Object|Array} element
         */
        this.changeElement = function (element) {
            IteratorGraph.call(this, element);
            this.prototype = Object.create(IteratorGraph.prototype);
        }
    }

    IteratorGraph.prototype = {
        /**
         * @description Place l'index avant le premier élément.
         */
        beforeFirst : function () {
            this.index = -1;
        },
        /**
         * @description Donne la première itération.
         * @returns {Object|Array|String|Number}
         */
        first : function () {
            this.reset();
            return this.current();
        },
        /**
         * @description Donne l'itération précédante.
         * @returns {Object|Array|String|Number}
         */
        previous : function () {
            return this.getElementAt(--this.index);
        },
        /**
         * @description Donne l'itération actuelle.
         * @returns {Object|Array|String|Number}
         */
        current : function () {
            return this.getElementAt(this.index);
        },
        /**
         * @description Donne la prochaine itération.
         * @returns {Object|Array|String|Number}
         */
        next : function () {
            return this.getElementAt(++this.index);
        },
        /**
         * @description Donne la dernière itération.
         * @returns {String|Object|Number|Array}
         */
        last : function () {
            this.index = this.size() - 1;

            return this.current();
        },
        /**
         * @description Place l'index après le dernier élément.
         */
        afterLast : function () {
            this.index = this.size();
        },
        /**
         * @description Réinitialise l'itérateur à sa première itération.
         */
        reset : function () {
            this.index = 0;
        },
        /**
         * @description Dit s'il y a encore au moins un élément avant le dernier que l'on a récupérer.
         * @returns {boolean}
         */
        hasPrevious : function () {
            return this.index > 0;
        },
        /**
         * @description Dit s'il y a encore au moins un élément après le dernier que l'on a récupérer.
         * @returns {boolean}
         */
        hasNext : function () {
            return this.index < this.size() - 1;
        },
        /**
         * @description Parcours tous les éléments en appelant callback pour chacun.
         * @param {Function} callback
         * @param {Function} [progressionCallback] Optionnel, appel de cette fonction fait tous les 1000 éléments. Give a percentage of accomplisshment each time it call.
         */
        forEach : function (callback, progressionCallback) {
            if (this.size() > 0) {
                var hasToCallProgressionCallback = progressionCallback && this.size() > 1000;
                this.beforeFirst();
                do {
                    callback(this.next());
                    if (hasToCallProgressionCallback && (this.index % 1000 === 0)) progressionCallback(this.index * 100 / this.size());
                } while (this.hasNext());
            }
        }
    };

    return IteratorGraph;
}());